// convert lines from a Diagram document into sinusoids.
//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: Wed Jun 25 15:33:07 GMT-0800 1997
// Last Modified: Mon Jun 30 09:44:52 GMT-0800 1997
// Filename:      .../sig/examples/line2sine/Vertex.cc
// Syntax:        C++
// $Smake:        cc -Wall -g -c %b.cc -I../../include && rm -f %b.o
//

#include <iostream.h>
#include <stdlib.h>
#include "Vertex.h"


//////////////////////////////
//
// Vertices::Vertices
//

Vertices::Vertices(void) {
   vertices.setSize(0);
   vertices.allowGrowth();
   sortedQ = 0;
}


//////////////////////////////
//
// Vertices::add
//

void Vertices::add(Vertex aVertex) {
   sortedQ = 0;
   vertices[vertices.getSize()] = aVertex;
}

void Vertices::add(int aTag, double xValue, double yValue) {
   Vertex aVertex(aTag, xValue, yValue);
   add(aVertex);
}

void Vertices::add(int aTag, doubleXY coords) {
   Vertex aVertex(aTag, coords.x, coords.y);
   add(aVertex);
}


//////////////////////////////
//
// Vertices::get
//

doubleXY Vertices::get(int aTag) {
   if (!sortedQ) {
      sort();
   }

   Vertex dummyVertex(aTag, 0, 0); 
   Vertex *vp;
   vp = (Vertex*) bsearch(&dummyVertex, vertices.getBase(), vertices.getSize(),
      sizeof(Vertex), vertexComp);
   if (vp == NULL) {
      cerr << "Error: could not find vertex " << aTag
           << " in list of vertices." << endl;
      exit(1);
   }
      
   doubleXY output(vp->x, vp->y);
   return output;
}
   


//////////////////////////////
//
// Vertices::print
//

void Vertices::print(void) {
   for (int i=0; i<vertices.getSize(); i++) {
      cout << "Vertex[" << vertices[i].tag << "] = (" 
           << vertices[i].x << ", " << vertices[i].y << ")" << endl;
   }
}



//////////////////////////////
//
// Vertices::sort
//

void Vertices::sort(void) {
   qsort(vertices.getBase(), vertices.getSize(), sizeof(Vertex),vertexComp);
   sortedQ = 1;
}



//////////////////////////////
//
// vertexComp -- compares two vertices.  If first has a lower tag number,
//	then returns a -1. If tags are the same, return 0. If the 
//	first tag is greater than the second, a 1 will be returned.
//

int vertexComp(const void* v1, const void* v2) {
   if ( ((Vertex*)v1)->tag < ((Vertex*)v2)->tag ) {
      return -1;
   } else if ( ((Vertex*)v1)->tag > ((Vertex*)v2)->tag ) {
      return 1;
   } else {
      return 0;
   }
}




